import csv


class CSVUtils:
    """This class simplifies the process of working with CSV files."""

    def __init__(self):
        pass

    def get_headers(self, infile):
        """
        Get a dictionary where k = header name, v = column index.
        :param infile: CSV file to read from
        :return: dictionary where k = header name, v = column index
        """
        headers_map = {}
        with open(infile) as csvfile:
            csvreader = csv.reader(csvfile)
            for line in csvreader:
                for i in range(len(line)):
                    headers_map[line[i]] = i
                break
        return headers_map

    def read_csv(self, infile, has_headers=True, selected_columns=None):
        """
        Read all lines in a CSV file to a list of lists.
        :param infile: CSV file to read from
        :param has_headers: whether or not the CSV file has a header line; default is True
        :param selected_columns: list of selected columns by either header name or index
        :return: tuple -- contents of CSV file as list of lists, headers dictionary
        """
        headers_map = None
        l1 = False
        if has_headers:
            headers_map = self.get_headers(infile)
            l1 = True
        lines = []
        with open(infile) as csvfile:
            csvreader = csv.reader(csvfile)
            for line in csvreader:
                if l1:
                    # skip reading header line
                    l1 = False
                    continue
                # make sure this line has contents
                if len(line):
                    # pull only selected columns
                    if selected_columns:
                        l = []
                        for c in selected_columns:
                            if isinstance(selected_columns[0], str):
                                # pull selected columns by header
                                l.append(line[headers_map[c]])
                            else:
                                # pull selected columns by index
                                l.append(line[c])
                        if len(l):
                            lines.append(l)
                    else:
                        if len(line):
                            lines.append(line)
        return lines, headers_map



from setuptools import find_packages, setup

# The text of the README file
with open("README.md") as file:
    README = file.read()

# This call to setup() does all the work
setup(
    name="mcneelat-pyutils",
    version="1.4.0",
    description="A collection of helpful Python utilities to simplify dealing with databases, Kafka, and more.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/tommym89/pyutils",
    author="Tommy McNeela",
    author_email="mcneelat@gmail.com",
    license="GPLv3",
    classifiers=[
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=["dnspython", "kafka-python", "psycopg2-binary", "python-ldap", "requests"],
)



from datetime import datetime as dt, timedelta
from mcneelat.pyutils.confutils import AbstractLogUtils
import requests


class ThreatStream(AbstractLogUtils):
    """Class containing handy methods common to working with the Anomali ThreatStream API."""

    """Map of general IOC categories to the most interesting IOC types."""
    ICATEGORY_MAP = {
        "domain": "apt_domain,c2_domain,exfil_domain,exploit_domain,mal_domain",
        "ip": "apt_ip,c2_ip,exfil_ip,exploit_ip,mal_ip",
        "url": "apt_url,c2_url,exfil_url,exploit_url,mal_url"
    }

    def __init__(self, api_user, api_key, min_confidence=50, last_modified_days=90,
                 results_limit=0, verbose=True):
        """
        Initialize class.
        :param api_user: API username
        :param api_key: API secret key
        :param min_confidence: minimum confidence for IOCs to look for
        :param last_modified_days: number of days ago IOCs must have been modified in order to include in results
        :param results_limit: limit of results from API queries; 0 = unlimited
        """
        self.base_url = "https://api.threatstream.com"
        self.intel_url_part = "/api/v2/intelligence/"
        self.creds_url_part = "username=%s&api_key=%s" % (api_user, api_key)
        self.min_confidence = min_confidence
        self.last_modified_days = last_modified_days
        self.results_limit = results_limit
        self.set_confidence(min_confidence)
        AbstractLogUtils.__init__(self, verbose)

    def set_confidence(self, min_confidence):
        """
        Set minimum confidence level.
        :param min_confidence: minimum confidence level on a scale of 0 to 100
        :return: None
        """
        self.min_confidence = min_confidence
        self.next_url_base = "%s?%s&status=active&confidence__gte=%i&limit=%i" % (
            self.intel_url_part, self.creds_url_part, self.min_confidence, self.results_limit
        )

    def is_threat(self, test_object):
        """
        Check if an indicator is malicious (i.e. an active IOC in ThreatStream)
        :param test_object: indicator to check
        :return: True if malicious, False if not found active in ThreatStream
        """
        result = self.get_ioc_details(test_object)
        return result is not None

    def get_ioc_details(self, test_object):
        """
        Get details for one specific IOC.
        :param test_object: object to search for
        :return: list of dictionaries (usually only one in the list) containing IOC details
        """
        last_modified = (dt.today() - timedelta(days=self.last_modified_days)).strftime("%Y-%m-%dT00:00:00Z")
        next_url = "%s&modified_ts__gte=%s&value=%s" % (self.next_url_base, last_modified, test_object)
        try:
            json_data = requests.get(self.base_url + next_url).json()
        except ValueError:
            return None
        try:
            json_data.get('objects')[0]
        except:
            return None
        return json_data.get('objects')

    def get_iocs(self, icategory, severity="high"):
        """
        Get a list of IOCs and their details from a specified category.
        :param icategory: domain, ip, or url
        :param severity: low, medium, high, very-high
        :return: list of IOCs and their details
        """
        last_modified = (dt.today() - timedelta(days=self.last_modified_days)).strftime("%Y-%m-%dT00:00:00Z")
        next_url = "%s&modified_ts__gte=%s&meta.severity__contains=%s&itype=%s" % (
            self.next_url_base, last_modified, severity, ThreatStream.ICATEGORY_MAP[icategory]
        )
        self.log("[*] Starting to gather IOCs...")
        results = []
        while next_url is not None and next_url != "null":
            try:
                json_data = requests.get(self.base_url + next_url).json()
            except ValueError:
                self.log("Warning, call to URL '%s' resulted in no JSON object response." % self.base_url + next_url)
                break
            results.extend(json_data.get('objects'))
            next_url = json_data.get('meta').get('next')
        return results



import ldap


class DirectoryActions:
    """
    Class containing methods to work with an LDAP server to perform login actions and resolve details about users.
    """

    def __init__(self, conf_data):
        """
        Constructor.
        :param conf_data: dictionary containing configuration data for the app
        """
        ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_NEVER)
        self.conf_data = conf_data

    def login(self, employeeid, password):
        """
        Perform login operation against an LDAP connection.
        :param employeeid: employee ID of user trying to log in
        :param password: password of user trying to log in
        :return: either details of user or False on failure
        """
        con = ldap.initialize(self.conf_data['LDAP_CONN_INFO']['server'])
        dn = self.conf_data['LDAP_CONN_INFO']['dn_base'] % employeeid
        try:
            con.simple_bind_s(dn, password)
            return DirectoryActions.get_userdetails(con, employeeid)
        except ldap.LDAPError as error_message:
            print(error_message)
            # return "Failed login on bind: dn=%s, pass=%s, error_message=%s" % (dn, password, error_message)
            return False

    def service_account_login(self):
        """
        Log into the LDAP server using a service account, which will allow us to search instead of only log in.
        :return: LDAP connection object or False on failure
        """
        con = ldap.initialize(self.conf_data['LDAP_CONN_INFO']['server'])
        try:
            con.simple_bind_s(self.conf_data['LDAP_SERVICE_ACCOUNT']['dn'],
                              self.conf_data['LDAP_SERVICE_ACCOUNT']['password'])
            return con
        except ldap.LDAPError as error_message:
            print(error_message)
            return False

    @staticmethod
    def search(con, search_term):
        """
        Generic search method against an LDAP connection.
        :param con: LDAP server connection object
        :param search_term: search term
        :return: dictionary of LDAP search results
        """
        base = ''
        scope = ldap.SCOPE_SUBTREE
        retrieve_attributes = None
        result_set = []
        timeout = 0
        try:
            result_id = con.search(base, scope, search_term, retrieve_attributes)
            while True:
                result_type, result_data = con.result(result_id, timeout)
                if result_data is []:
                    break
                else:
                    if result_type == ldap.RES_SEARCH_ENTRY:
                        result_set.append(result_data)
            if len(result_set) == 0:
                print("Empty result set: search-term=%s" % search_term)
                # return "Empty result set: search-term=%s" % search_term
                return False
            return result_set[0][0][1]
        except ldap.LDAPError as error_message:
            print(error_message)
            # return error_message
            return False

    @staticmethod
    def get_userdetails(con, employeeid):
        """
        Get details about the user with the specified employee ID.
        :param con: LDAP server connection object
        :param employeeid: user employee ID
        :return: dictionary with user details or false on failure
        """
        dir_result = DirectoryActions.search(con, "cn=%s" % employeeid)
        if not dir_result:
            return False
        if dir_result['employeeStatus'][0] != 'Active':
            return False
        return dir_result
